require 'spec_helper'

describe 'droneci', :type => 'class' do

  context "On a Debian OS with no package name specified" do
    let(:facts) {{ :osfamily => 'Debian' }}
    it {
      should contain_package('drone').with( { 'name' => 'drone' } )
      should contain_service('drone').with( { 'name' => 'drone' } )
    }
  end

  context "On a RedHat OS with no package name specified" do
    let(:facts) {{ :osfamily => 'RedHat' }}
    it {
      should contain_package('drone').with( { 'name' => 'drone' } )
      should contain_service('drone').with( { 'name' => 'drone' } )
    }
  end

  context "On an unknown OS with no package name specified" do
    let(:facts) {{ :osfamily => 'Darwin' }}
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  context "If only ssl_key is specified the compilation should fail" do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do
      {
        :ssl_key => 'testcert'
      }
    end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  context "If only ssl_cert is specified the compilation should fail" do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do
      {
        :ssl_cert => 'testcert'
      }
    end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  context "If not specified, default server port should be configured" do
    let(:facts) {{ :osfamily => 'RedHat' }}
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[server\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^port=\"\:8080\"/)
    }
  end

  context "If specified, selected port should be configured" do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let(:params) {{ :server_port => ':9000'}}
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[server\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^port=\"\:9000\"/)
    }
  end

  context "Should contain SSL key and cert if specified" do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do
      {
        :ssl_key => 'testkey',
        :ssl_cert => 'testcert'
      }
    end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[server.ssl\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^key=\"testkey\"/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^cert=\"testcert\"$/)
    }
  end

  context 'If no database is specified, Sqlite is selected' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[database\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^driver=\"sqlite3\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^datasource=\"\/var\/lib\/drone\/drone.sqlite\"$/)
    }
  end

  context 'If a database is specified, should contain those values' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do
    {
      :db_driver => 'mysql',
      :db_datasource => '/dummy/data/source'
    }
    end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[database\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^driver=\"mysql\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^datasource=\"\/dummy\/data\/source\"$/)
    }
  end

  # Testing Github

  context 'If all the paramenters are well defined' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_client => 'myclient',
      :github_secret => 'mysecret',
      :github_orgs => ["drone", "docker"],
      :github_open => true
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[github\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^client=\"myclient\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^secret=\"mysecret\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^orgs=\[\"drone\", \"docker\"\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^open=true$/)
    }
  end

  context 'If github_org is not an array' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_client => 'myclient',
      :github_secret => 'mysecret',
      :github_orgs => 'notanarray',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  context 'If github_open is not an boolean' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_client => 'myclient',
      :github_secret => 'mysecret',
      :github_open => 'notabool',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  # Testing Github Enterprise

  context 'If all the paramenters are well defined' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_ent_client => 'myclient',
      :github_ent_secret => 'mysecret',
      :github_ent_api => 'myapi',
      :github_ent_url => 'myurl',
      :github_ent_orgs => ["drone", "docker"],
      :github_ent_open => true,
      :github_ent_private_mode => true,
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[github_enterprise\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^client=\"myclient\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^secret=\"mysecret\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^api=\"myapi\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^url=\"myurl\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^orgs=\[\"drone\", \"docker\"\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^open=true$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^private_mode=true$/)
    }
  end

  context 'If github_ent_org is not an array' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_ent_client => 'myclient',
      :github_ent_secret => 'mysecret',
      :github_ent_orgs => 'notanarray',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  context 'If github_ent_open is not an boolean' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_ent_client => 'myclient',
      :github_ent_secret => 'mysecret',
      :github_ent_open => 'notabool',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  context 'If github_ent_private_mode is not an boolean' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :github_ent_client => 'myclient',
      :github_ent_secret => 'mysecret',
      :github_ent_private_mode => 'notabool',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  # Testing Bitbucket

  context 'If all the paramenters are well defined' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :bitbucket_client => 'myclient',
      :bitbucket_secret => 'mysecret',
      :bitbucket_open => true
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[bitbucket\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^client=\"myclient\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^secret=\"mysecret\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^open=true$/)
    }
  end

  context 'If bitbucket_open is not an boolean' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :bitbucket_client => 'myclient',
      :bitbucket_secret => 'mysecret',
      :bitbucket_open => 'notabool',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  # Testing Gitlab

  context 'If all the paramenters are well defined' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :gitlab_url => 'myurl',
      :gitlab_client => 'myclient',
      :gitlab_secret => 'mysecret',
      :gitlab_open => true,
      :gitlab_skip_verify => true
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[gitlab\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^client=\"myclient\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^secret=\"mysecret\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^open=true$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^skip_verify=true$/)
    }
  end

  context 'If gitlab_open is not an boolean' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :gitlab_client => 'myclient',
      :gitlab_secret => 'mysecret',
      :gitlab_open => 'notabool',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  # Testing Gogs

  context 'If all the paramenters are well defined' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :gogs_url => 'myurl',
      :gogs_secret => 'mysecret',
      :gogs_open => true
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[gogs\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^url=\"myurl\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^secret=\"mysecret\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^open=true$/)
    }
  end

  context 'If gogs_open is not an boolean' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :gogs_url => 'myurl',
      :gogs_secret => 'mysecret',
      :gogs_open => 'notabool',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end

  # Testing Smtp

  context 'Testing smtp settings' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :smtp_host => 'dummyhost',
      :smtp_port => 'dummyport',
      :smtp_from => 'dummyfrom',
      :smtp_user => 'dummyuser',
      :smtp_pass => 'dummypass',
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[smtp\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^host=\"dummyhost\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^port=\"dummyport\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^from=\"dummyfrom\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^user=\"dummyuser\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^pass=\"dummypass\"$/)
    }
  end

  # Testing Docker

  context 'Testing docker settings' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :docker_cert => 'mycert',
      :docker_key => 'mykey'
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[docker\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^cert=\"mycert\"$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^key=\"mykey\"$/)
    }
  end

  # Testing Workers

  context 'Testing worker settings' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :worker_nodes => ["a", "b"],
    } end
    it {
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^\[worker\]$/)
      should contain_file('/etc/drone/drone.toml') \
        .with_content(/^nodes=\[\"a\"\, \"b\"\]$/)
    }
  end
  
  context 'If worker_nodes is not an array' do
    let(:facts) {{ :osfamily => 'RedHat' }}
    let :params do {
      :worker_nodes => 'notanarray',
    } end
    it {
      expect { should raise_error(Puppet::Error) }
    }
  end
end

